#!/bin/sh
#
# Purpose:
#   Splits study to two or more parts
#
#
# Synopsis:
#   splitstudy.sh <study>
#
# Arguments:
#      study     : Study Instance UID of the study to be split
#
#
# splistudy.sh will look for new study files. The file name of new study files will
# look like:
#
# -  $TMP/$study_list_$i (where 0 <= i < number of parts)
#
# The first line of the new study files will contain the Accession Number
# of the new study, the remainder of the file is expected as an object list
# to be copied from original study to the new study.
#
# eg:
#  | ACCNO=ACN-00001
#  | NM.1.2.84.73483478.367436.1
#  | NM.1.2.84.73483478.367436.2
#  | NM.1.2.84.73483478.367436.3
#  | NM.1.2.84.73483478.367436.4
#  | NM.1.2.84.73483478.367436.5
#  | NM.1.2.84.73483478.367436.6
#  | NM.1.2.84.73483478.367436.7
#  | NM.1.2.84.73483478.367436.8
#
# For each new study a directory will be created and objects listed in
# list files will be copied from original study directory to these new
# directories
#
# A new Study Instance UID for each new study and new SOP Instance UID for
# each object in new studies will be generated, and all object in new studies
# will be edited to pick up new Study Instance UIDs, SOP Instance UIDs and
# Accession numbers.
#
# New studies will be moved to data/dicom and get registered.
#
# ENDDOC
#
#  Last Modified by : $Author$
#  Last Modified at : $Date$
#  RCS File Name    : $RCSfile$
#  RCS Revision     : $Revision$
#  Locked by        : $Locker$
#  Symbolic name    : $Name$
#  State            : $State$
#

set > /home/medsrv/tmp/splitenv.1

. $HOME/etc/setenv.sh 2>/home/medsrv/tmp/setenv.err

set > /home/medsrv/tmp/splitenv.2

LOGGER="/usr/bin/logger"
TAG="splitstudy[$$]"

STUDY=$1

CWD=`pwd`
LIST_FILES=""
NEWSTUDIES=""

Date=`date +"%Y%m%d%H%M%S"`


# recieve in environment variable
[ -z "$LOG_USER" ] && LOG_USER="-"

# include studyinfo.shi to get study data from database for log messages
if [ -f "$CASES_HOME/studyinfo.shi" ] ; then
	study=$STUDY
	. $CASES_HOME/studyinfo.shi
fi

[ -z "$DB_PNAME" ] && DB_PNAME="-"
[ -z "$DB_MODALITY" ] && DB_MODALITY="-"

REFERENCEID="splitstudy.$$"

DERIVE_INFO_FILE="$TMP/split-deriveInfo.$$"


# prints error message to standard error
# end exits with error code 1
#
# $1  - error message to be printed
#
error_message_()
{
	LOG_ERROR=`echo "$1" | sed 's/[^a-zA-Z0-9\.\-_+]/_/g'`
	[ -z "$STUDY" ] && STUDY="-"
	$LOGGER -p $LOG_OPER_FACILITY.error -t "$TAG" "ERROR STUDY split $LOG_USER - $STUDY $DB_PNAME $DB_MODALITY - - - - - - - $LOG_ERROR"
	$LOG_HOME/logintodatabase.sh -p "ERROR" -l "STUDY" -a "split" -u "$LOG_USER" -s "$STUDY" -p "$DB_PNAME" -m "$LOG_ERROR" 
	echo $1 >&2
	cd $CWD

	exit 1
}


# cleans up temporary changes
# and calls error_message_ if $1 is
# specified
#
# [$1]  - error message to be printed
#
cleanup_()
{
	# remove splitstudy.$$ reference from STUDYDB reference counter of the study
	$UTILS_HOME/bin/refcount -r $REFERENCEID -t STUDYDB -i "$STUDY" >>$LOG/refcount.error 2>&1

	for i in $NEWSTUDIES ; do
		[ -d "$TMP/$i" ] && rm -fr "$TMP/$i"
	done

	[ -f "${DERIVE_INFO_FILE}" ] && rm -f "${DERIVE_INFO_FILE}"

	if [ -n "$1" ] ; then
		# lock should only be released in case of error
		# in case of success the lock will be released by obsolete task
		[ -n "${STUDY}" ] && rm ${LOCK_VAR}/r_${STUDY} 2>/dev/null
		error_message_ "$1"
	fi

	cd $CWD
	return 0
}



#########################################################################################
#
# Main
#
#########################################################################################


[ -z "$1" ] && error_message_ "Missing argument: Study Instance UID of original study"


studydir=`$REPOSITORY_HANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $STUDY`
[ $? -ne 0 ] && error_message_ "Failed to locate study in the repository"

cd $TMP

[ $? -ne 0 ] && error_message_ "Can not change current directory to $TMP"

COUNTER=0

# collect list files of new studies

while [ -f "${STUDY}_list_${COUNTER}" ] ; do
	LIST_FILES="$LIST_FILES ${STUDY}_list_${COUNTER}"
	COUNTER=$(( $COUNTER + 1 ))
done

[ -z "$LIST_FILES" ] && error_message_ "No list files found"

NUM_NEW_STUDIES=`echo $LIST_FILES| wc -w | awk '{print $1}'`

[ $NUM_NEW_STUDIES -lt 2 ] && error_message_ "At least two new study list file required"


# add splitstudy.$$ reference to STUDYDB reference counter of the study to be split to make sure the
# study will not be deleted before the split completes
$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "$UTILS_HOME/bin/refcount -E -a $REFERENCEID -t STUDYDB -i \"$STUDY\" "

$UTILS_HOME/bin/refcount -E -a $REFERENCEID -t STUDYDB -i "$STUDY"
[ $? -ne 0 ] && error_message_ "Could not add reference to study $STUDY"


#
# Create new studies in the $TMP directory
#
IS_FIRST="true"
SELECTED_STUDY=""
REPORTS_COPIED="false"

for i in $LIST_FILES ; do
	ACCNO=`head -n 1 $i | awk -vFS="=" '($1 == "ACCNO") {print $2}'`
	[ -z "$ACCNO" ] && cleanup_ "$1 is bad formatted: contains no accession number"

	COPY_REPORTS=`tail -n +2 $i |head -n 1 | awk -vFS="=" '($1 == "COPY_REPORTS") {print $2}'`

	NEWSTUDY=`$UTILS_HOME/bin/createUID study`
	[ -z "$NEWSTUDY" ] && cleanup_ "Can not create new Study Instance UID"

	if [ "true" = "$IS_FIRST" ] ; then
		SELECTED_STUDY=$NEWSTUDY
		echo "NewStudyUID=\"$NEWSTUDY\"" > ${TMP}/moveto.${STUDY}
	fi

	$REPOSITORY_HANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM $NEWSTUDY >/dev/null
	[ $? -lt 4 ] && cleanup_ "Study Instance UID $NEWSTUDY already exists"
	#0 means OK, 1 means dirty, 2 means conflict, in these cases it already exists

	mkdir "$TMP/$NEWSTUDY"
	[ $? -ne 0 ] && cleanup_ "Can not create $TMP/$NEWSTUDY directory"

	mkdir "$TMP/$NEWSTUDY/src"
	[ $? -ne 0 ] && cleanup_ "Can not create $TMP/$NEWSTUDY/src directory"

	mkdir "$TMP/$NEWSTUDY/dest"
	[ $? -ne 0 ] && cleanup_ "Can not create $TMP/$NEWSTUDY/dest directory"


	NEWSTUDIES="$NEWSTUDIES $NEWSTUDY"

	FILELIST=$TMP/$NEWSTUDY/filelist

	echo "${STUDY} split ${NEWSTUDY}" >> ${DERIVE_INFO_FILE}

	tail -n +3 $i | while read fname ; do
		cp $studydir/$fname $TMP/$NEWSTUDY/src/
		[ $? -ne 0 ] && cleanup_ "Error copying $fname from $studydir/$fname to $TMP/$NEWSTUDY/src/"
		echo "$TMP/$NEWSTUDY/src/$fname" >> $FILELIST
	done
	# while read starts new processes
	[ $? -ne 0 ] && cleanup_ "Error creating new DICOM objects"

	ReportFiles=` ls $studydir/ | grep -e ^SRt -e PbD`

	# Copy PbR file(s)
	for pbr in `ls $studydir/ | grep ^PbR` ; do
		res0=0;
		if [ "true" != "$COPY_REPORTS" -a -n "$ReportFiles" ] ; then
			# There were report files, but no report would be kept for this study - reset status in PbR
			$DCVIEWER_HOME/dcmod -o "$TMP/$NEWSTUDY/src/$pbr" -t "(f215,1002)" -r "IS" -v "0" "$studydir/$pbr"
			res0=$?
		else
			cp $studydir/$pbr $TMP/$NEWSTUDY/src/
			res0=$?
		fi
		
		[ $res0 -ne 0 ] && cleanup_ "Error copying $fname from $studydir/$pbr to $TMP/$NEWSTUDY/src/"
		echo "$TMP/$NEWSTUDY/src/$pbr" >> $FILELIST
	done

	if [ "true" = "$COPY_REPORTS" ] ; then
		# Copy SRt and PbD files
		for reportf in `ls $studydir/ | grep -e ^SRt -e PbD` ; do
			cp $studydir/$reportf $TMP/$NEWSTUDY/src/
			[ $? -ne 0 ] && cleanup_ "Error copying $fname from $studydir/$reportf to $TMP/$NEWSTUDY/src/"
			echo "$TMP/$NEWSTUDY/src/$reportf" >> $FILELIST
		done
		REPORTS_COPIED="true"
	fi

	MODFILE="$TMP/$NEWSTUDY/modfile"
	UIDLIST="$TMP/$NEWSTUDY/uidlist"

	cat << ENDOFMODFILE > $MODFILE.dmp
(0008,0050) SH [$ACCNO]
(0020,000d) UI [$NEWSTUDY]
ENDOFMODFILE

	$DCMTK_HOME/bin/dump2dcm $MODFILE.dmp $MODFILE
	[ $? -ne 0 ] && cleanup_ "Error creating modfile from $MODFILE"

	echo "Replace $STUDY by $NEWSTUDY" > $UIDLIST

	if [ "true" = "$IS_FIRST" ] ; then
		IS_FIRST="false"
		cp "$MODFILE" ${TMP}/moveto.${SELECTED_STUDY}.mod
		$UTILS_HOME/bin/copyDicomFiles -d $TMP/$NEWSTUDY/dest -f $FILELIST -m $MODFILE -l $UIDLIST >${TMP}/moveto.${NEWSTUDY}.uids
	else
		$UTILS_HOME/bin/copyDicomFiles -d $TMP/$NEWSTUDY/dest -f $FILELIST -m $MODFILE -l $UIDLIST > $TMP/$NEWSTUDY/out
	fi
	
	[ $? -ne 0 ] && cleanup_ "Error copying study files to $TMP/$NEWSTUDY/dest"
done


# For setting staged devices for new studies
StagedDevices=""
ForwardDevices=""
Folder=""
. $CASES_HOME/includeStorestate.shi "$STUDY"


# Create storestate arguments for copying Folder, StagedDevices and ForwardDevices
# attributes into the new storestate

StoreStateArgs=""
for dev in $StagedDevices ; do
	StoreStateArgs="$StoreStateArgs --setstaged=\"$dev\""
done

for dev in $ForwardDevices ; do
	StoreStateArgs="$StoreStateArgs --fwddevice=\"$dev\""
done

[ -n "$Folder" ] && StoreStateArgs="$StoreStateArgs --folder=\"$Folder\""

#check that all file was copied to NEW_STUDY_DIR
#if not it calls moveLateObject.sh
for file in `ls -v $studydir/` ; do
	[ "$file" = "storestate.rec" ] && continue
	[ "${file#PbR.}" != "$file" ] && continue
	[ "${file#PbD.}" != "$file" ] && continue
	[ "${file#SRt}" != "$file" ] && continue

	found="false"
	for study in $NEWSTUDIES ; do
		$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "check if ${TMP}/${study}/src/${file} exists"
		if [ -e "${TMP}/${study}/src/${file}" ] ; then
			found="true"
			$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "${TMP}/${study}/src/${file} exists"
			break
		fi
	done

	if [ "true" = "$found" ] ; then
		continue
	fi

	if [ ! -d "${TMP}/obsolete/${STUDY}" ] ; then
		mkdir -p ${TMP}/obsolete/${STUDY}
	fi
	
	mv $studydir/$file ${TMP}/obsolete/${STUDY}/${file}
	$LOGGER -p $LOG_FACILITY.info -t "$TAG" "mv $studydir/$file to ${TMP}/obsolete/${STUDY}/${file} was moved because split has started"

	$TASKD_HOME/createtaskfileapp -p "MOVE_LATE_OBJECT" -t "sh" -c "$CASES_HOME/moveLateObjects.sh $file $STUDY"
done
										

# move studies to IMAGE_TMP
# and start registering new studies

TaskUser="$LOG_USER"
[ "-" = "$TaskUser" ] && TaskUser=""

for study in $NEWSTUDIES ; do
	# Create .info/storestate.rec file with StagedDevices and ForwardDevices copied from originals
	StudyStoreStateArgs="--study=\"${study}\" --processmode=\"Process\" --owner=\"yes\" --derived=\"split\" $StoreStateArgs"
	eval ${UTILS_HOME}/bin/storestate $StudyStoreStateArgs

	$LOGGER -p $LOG_OPER_FACILITY.info -t "$TAG" "INFO STUDY split $LOG_USER - $study $DB_PNAME $DB_MODALITY - - - - - - - study_created"
	$LOG_HOME/logintodatabase.sh -p "INFO" -l "STUDY" -a "split" -u "$LOG_USER" -s "$study" -p "$DB_PNAME" -m "study_created" 

	# Create derivation info file in .info of the new study
	newStudyDir=`$REPOSITORY_HANDLER_HOME/bin/locateResource $REPOSITORY_DATA_DICOM ${study}`
	if [ $? -eq 0 -a -n "${newStudyDir}" ] ; then
		cp "${DERIVE_INFO_FILE}" "${newStudyDir}/.info/derivationInfo"
	fi

	# make sure PbRs get registered first
	for file in `ls $TMP/$study/dest/ | grep ^PbR` ; do
		tempFile="$IMAGE_TMP/${study}_${file}_$$"
		mv "$TMP/$study/dest/$file" "${tempFile}"

		$UTILS_HOME/bin/storeDICOMFile -f "$file" -s "$study" -t "${tempFile}" -D "DELTA_STORESCP_REG_SPLIT_PBR" -U "${TaskUser}" -N "splitting study"
		if [ $? -ne 0 ] ; then
			# StorescpReg task could not be placed
			rm "$tempFile}" 2>/dev/null
		fi
	done

	# register remaining (non-PbR) objects
	for file in `ls $TMP/$study/dest/ | grep -v ^PbR` ; do
		tempFile="$IMAGE_TMP/${study}_${file}_$$"
		mv "$TMP/$study/dest/$file" "${tempFile}"

		$UTILS_HOME/bin/storeDICOMFile -f "$file" -s "$study" -t "${tempFile}" -D "DELTA_STORESCP_REG_SPLIT" -U "${TaskUser}" -N "splitting study"
		if [ $? -ne 0 ] ; then
			# StorescpReg task could not be placed
			rm "$tempFile}" 2>/dev/null
		fi
	done

	rmdir $TMP/$study/dest 2>&1
done

#infos to moveto. file
echo "UIDFile=\"${TMP}/moveto.${SELECTED_STUDY}.uids\"" >> ${TMP}/moveto.${STUDY}
echo "ModFile=\"${TMP}/moveto.${SELECTED_STUDY}.mod\"" >> ${TMP}/moveto.${STUDY}

# Collect shortcuts to original study
shortCuts=`echo "select STYIUID from $STUDYTBL where SRCSTUDY='${STUDY}' AND STYIUID<>'${STUDY}';" |
${MYSQL_HOME}/bin/mysql -N $DBNAME`


# Create Shortcuts for new studies if any
for shortcut in $shortCuts ; do
	# copy shortcut for new studies
	
	for study in $NEWSTUDIES ; do
		$LOGGER -p $LOG_FACILITY.debug -t "$TAG" "${SR_HOME}/bin/createLink -M $shortcut -s $study > tmp/out.$$.$study 2> tmp/err.$$.$study"
		${SR_HOME}/bin/createLink -u "${LOG_USER}" -M $shortcut -s $study > "$TMP/out.$$.$study" 2> "$TMP/err.$$.$study"
	done
	
	# obsolete old shortcut
	$TASKD_HOME/createtaskfileapp -p "RM_STUDY" -t "class" -c "${CASES_HOME#$HOME/component/}.RmStudy -s \"$shortcut\""
done 

# obsolete original study
$TASKD_HOME/createtaskfileapp -p "OBSOLETE_STUDY" -t "class" -c "${CASES_HOME#$HOME/component/}.ObsoleteStudy -s $STUDY"

$LOGGER -p $LOG_OPER_FACILITY.info -t "$TAG" "INFO STUDY split $LOG_USER - $STUDY $DB_PNAME $DB_MODALITY - - - - - - - study_splitted_into_${NUM_NEW_STUDIES}_parts"
$LOG_HOME/logintodatabase.sh -p "INFO" -l "STUDY" -a "split" -u "$LOG_USER" -s "$STUDY" -p "$DB_PNAME" -m "study_splitted_into_${NUM_NEW_STUDIES}_parts" 

if [ "false" = "$REPORTS_COPIED" ] ; then
	$LOGGER -p $LOG_OPER_FACILITY.info -t "$TAG" "INFO STUDY split $LOG_USER - $STUDY $DB_PNAME $DB_MODALITY - - - - - - - study_reports_deleted"
	$LOG_HOME/logintodatabase.sh -p "INFO" -l "STUDY" -a "split" -u "$LOG_USER" -s "$STUDY" -p "$DB_PNAME" -m "study_reports_deleted" 
fi


cleanup_

exit 0
